Module: ClosureTree::NumericDeterministicOrdering

Extended by:
ActiveSupport::Concern
Defined in:
lib/closure_tree/numeric_deterministic_ordering.rb

Instance Method Summary collapse

Instance Method Details

#_ct_reorder_children(minimum_sort_order_value = nil) ⇒ Object



24
25
26
# File 'lib/closure_tree/numeric_deterministic_ordering.rb', line 24

def _ct_reorder_children(minimum_sort_order_value = nil)
  _ct.reorder_with_parent_id(_ct_id, minimum_sort_order_value)
end

#_ct_reorder_prior_siblings_if_parent_changedObject



12
13
14
15
16
17
# File 'lib/closure_tree/numeric_deterministic_ordering.rb', line 12

def _ct_reorder_prior_siblings_if_parent_changed
  if public_send(:saved_change_to_attribute?, _ct.parent_column_name) && !@was_new_record
    was_parent_id = public_send(:attribute_before_last_save, _ct.parent_column_name)
    _ct.reorder_with_parent_id(was_parent_id)
  end
end

#_ct_reorder_siblings(minimum_sort_order_value = nil) ⇒ Object



19
20
21
22
# File 'lib/closure_tree/numeric_deterministic_ordering.rb', line 19

def _ct_reorder_siblings(minimum_sort_order_value = nil)
  _ct.reorder_with_parent_id(_ct_parent_id, minimum_sort_order_value)
  reload unless destroyed?
end

#add_sibling(sibling, add_after = true) ⇒ Object



113
114
115
116
117
118
119
120
121
122
123
124
125
126
127
128
129
130
131
132
133
134
135
136
137
138
139
140
141
142
143
144
145
146
147
148
149
150
151
152
# File 'lib/closure_tree/numeric_deterministic_ordering.rb', line 113

def add_sibling(sibling, add_after = true)
  fail "can't add self as sibling" if self == sibling

  if _ct.dont_order_roots && parent.nil?
    raise ClosureTree::RootOrderingDisabledError.new("Root ordering is disabled on this model")
  end

  # Make sure self isn't dirty, because we're going to call reload:
  save

  _ct.with_advisory_lock do
    prior_sibling_parent = sibling.parent
    reorder_from_value = if prior_sibling_parent == self.parent
      [self.order_value, sibling.order_value].compact.min
    else
      self.order_value
    end

    sibling.order_value = self.order_value
    sibling.parent = self.parent
    sibling._ct_skip_sort_order_maintenance!
    sibling.save # may be a no-op

    _ct_reorder_siblings(reorder_from_value)

    # The sort order should be correct now except for self and sibling, which may need to flip:
    sibling_is_after = self.reload.order_value < sibling.reload.order_value
    if add_after != sibling_is_after
      # We need to flip the sort orders:
      self_ov, sib_ov = self.order_value, sibling.order_value
      update_order_value(sib_ov)
      sibling.update_order_value(self_ov)
    end

    if prior_sibling_parent != self.parent
      prior_sibling_parent.try(:_ct_reorder_children)
    end
    sibling
  end
end

#append_child(child_node) ⇒ Object



89
90
91
# File 'lib/closure_tree/numeric_deterministic_ordering.rb', line 89

def append_child(child_node)
  add_child(child_node)
end

#append_sibling(sibling_node) ⇒ Object



105
106
107
# File 'lib/closure_tree/numeric_deterministic_ordering.rb', line 105

def append_sibling(sibling_node)
  add_sibling(sibling_node, true)
end

#prepend_child(child_node) ⇒ Object



93
94
95
96
97
98
99
100
101
102
103
# File 'lib/closure_tree/numeric_deterministic_ordering.rb', line 93

def prepend_child(child_node)
  child_node.order_value = -1
  child_node.parent = self
  child_node._ct_skip_sort_order_maintenance!
  if child_node.save
    _ct_reorder_children
    child_node.reload
  else
    child_node
  end
end

#prepend_sibling(sibling_node) ⇒ Object



109
110
111
# File 'lib/closure_tree/numeric_deterministic_ordering.rb', line 109

def prepend_sibling(sibling_node)
  add_sibling(sibling_node, false)
end

#self_and_descendants_preorderedObject



28
29
30
31
32
33
34
35
36
37
38
39
40
41
42
43
# File 'lib/closure_tree/numeric_deterministic_ordering.rb', line 28

def self_and_descendants_preordered
  # TODO: raise NotImplementedError if sort_order is not numeric and not null?
  join_sql = "    JOIN \#{_ct.quoted_hierarchy_table_name} anc_hier\n      ON anc_hier.descendant_id = \#{_ct.quoted_hierarchy_table_name}.descendant_id\n    JOIN \#{_ct.quoted_table_name} anc\n      ON anc.\#{_ct.quoted_id_column_name} = anc_hier.ancestor_id\n    JOIN \#{_ct.quoted_hierarchy_table_name} depths\n      ON depths.ancestor_id = \#{_ct.quote(self.id)} AND depths.descendant_id = anc.\#{_ct.quoted_id_column_name}\n  SQL\n\n  self_and_descendants\n    .joins(join_sql)\n    .group(\"\#{_ct.quoted_table_name}.\#{_ct.quoted_id_column_name}\")\n    .reorder(self.class._ct_sum_order_by(self))\nend\n"